@@ -1,5 +1,6 @@
 /*
- * Copyright © 2009-2021 Inria.  All rights reserved.
+ * SPDX-License-Identifier: BSD-3-Clause
+ * Copyright © 2009-2023 Inria.  All rights reserved.
  * Copyright © 2009-2012 Université Bordeaux
  * Copyright © 2009-2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
@@ -35,11 +36,13 @@ static int logical = 1;
 static int single_ancestor = 0;
 #define NO_ONLY_PID -1
 static long only_pid = NO_ONLY_PID;
+static long children_of_pid = NO_ONLY_PID;
 static long only_uid;
 static int json_server = 0;
 static int json_port = JSON_PORT;
 static FILE *json_output = NULL;
 static int verbose = 0;
+static FILE *lstopo_misc_output = NULL;
 
 void usage(const char *name, FILE *where)
 {
@@ -47,6 +50,7 @@ void usage(const char *name, FILE *where)
   fprintf (where, "Options:\n");
   fprintf (where, "  -a                 Show all processes, including those that are not bound\n");
   fprintf (where, "  --pid <pid>        Only show process of pid number <pid>\n");
+  fprintf (where, "  --children-of-pid <pid> Only show process of pid number <pid> and its children\n");
   fprintf (where, "  --name <name>      Only show processes whose name contains <name>\n");
 #ifdef HWLOC_LINUX_SYS
   fprintf (where, "  --uid <uid>        Only show processes of the user with the given uid\n");
@@ -64,6 +68,7 @@ void usage(const char *name, FILE *where)
   fprintf (where, "  --pid-cmd <cmd>    Append the output of <cmd> <pid> to each PID line\n");
   fprintf (where, "  --short-name       Show only the process short name instead of the path\n");
   fprintf (where, "  --disallowed       Include objects disallowed by administrative limitations\n");
+  fprintf (where, "  --lstopo-misc <file>  Output Misc object to be given to lstopo --misc-from <file>\n");
   fprintf (where, "  --json-server      Run as a JSON server\n");
   fprintf (where, "  --json-port <n>    Use port <n> for JSON server (default is %d)\n", JSON_PORT);
   fprintf (where, "  -v --verbose       Increase verbosity\n");
@@ -133,6 +138,45 @@ static void print_process(hwloc_topology_t topology,
 	print_task(topology, proc->threads[i].tid, proc->threads[i].name, proc->threads[i].cpuset, NULL, 1);
 }
 
+static void print_process_lstopo_misc(hwloc_topology_t topology __hwloc_attribute_unused,
+                                      struct hwloc_ps_process *proc)
+{
+  /* sort of similar to foreach_process_cb() in lstopo.c */
+  char name[100];
+  char *s;
+  unsigned i;
+
+  snprintf(name, sizeof(name), "%ld", proc->pid);
+  if (*proc->name)
+    snprintf(name, sizeof(name), "%ld %s", proc->pid, proc->name);
+  hwloc_bitmap_asprintf(&s, proc->cpuset);
+  fprintf(lstopo_misc_output,
+          "name=%s\n"
+          "cpuset=%s\n"
+          "subtype=Process\n"
+          "\n",
+          name, s);
+  free(s);
+
+  if (proc->nthreads)
+    for(i=0; i<proc->nthreads; i++)
+      if (proc->threads[i].cpuset) {
+	char task_name[150];
+	if (*proc->threads[i].name)
+	  snprintf(task_name, sizeof(task_name), "%s %li %s", name, proc->threads[i].tid, proc->threads[i].name);
+	else
+	  snprintf(task_name, sizeof(task_name), "%s %li", name, proc->threads[i].tid);
+        hwloc_bitmap_asprintf(&s, proc->threads[i].cpuset);
+        fprintf(lstopo_misc_output,
+                "name=%s\n"
+                "cpuset=%s\n"
+                "subtype=Thread\n"
+                "\n",
+                task_name, s);
+        free(s);
+      }
+}
+
 static void print_process_json(hwloc_topology_t topology,
 			       struct hwloc_ps_process *proc)
 {
@@ -197,7 +241,7 @@ static void foreach_process_cb(hwloc_topology_t topology,
   const char *pidcmd = cbdata;
 
   /* don't print anything if the process isn't bound and if no threads are bound and if not showing all */
-  if (!proc->bound && (!proc->nthreads || !proc->nboundthreads) && !show_all && !only_name)
+  if (!proc->bound && (!proc->nthreads || !proc->nboundthreads) && !show_all && !only_name && children_of_pid == NO_ONLY_PID)
     return;
 
   if (pidcmd)
@@ -205,6 +249,8 @@ static void foreach_process_cb(hwloc_topology_t topology,
 
   if (json_output)
     print_process_json(topology, proc);
+  else if (lstopo_misc_output)
+    print_process_lstopo_misc(topology, proc);
   else
     print_process(topology, proc);
 }
@@ -212,7 +258,11 @@ static void foreach_process_cb(hwloc_topology_t topology,
 static int run(hwloc_topology_t topology, hwloc_const_bitmap_t topocpuset,
 	       unsigned long psflags, char *pidcmd)
 {
-  if (only_pid == NO_ONLY_PID) {
+  if (children_of_pid != NO_ONLY_PID) {
+    /* show children */
+    return hwloc_ps_foreach_child(topology, topocpuset, children_of_pid, foreach_process_cb, pidcmd, psflags, only_name, only_uid);
+
+  } else if (only_pid == NO_ONLY_PID) {
     /* show all */
     return hwloc_ps_foreach_process(topology, topocpuset, foreach_process_cb, pidcmd, psflags, only_name, only_uid);
 
@@ -235,6 +285,8 @@ static int run(hwloc_topology_t topology, hwloc_const_bitmap_t topocpuset,
 
       if (json_output)
 	print_process_json(topology, &proc);
+      else if (lstopo_misc_output)
+        print_process_lstopo_misc(topology, &proc);
       else
 	print_process(topology, &proc);
     }
@@ -317,6 +369,7 @@ run_json_server(hwloc_topology_t topology, hwloc_const_bitmap_t topocpuset)
 
       only_name = NULL;
       only_pid = NO_ONLY_PID;
+      children_of_pid = NO_ONLY_PID;
       current = req;
       while (*current) {
 	if (!strncmp(current, "lastcpulocation ", 16)) {
@@ -338,6 +391,10 @@ run_json_server(hwloc_topology_t topology, hwloc_const_bitmap_t topocpuset)
 	  psflags |= HWLOC_PS_FLAG_THREADS;
 	  show_all = 1;
 	  break;
+	} else if (!strncmp(current, "childrenofpid=", 14)) {
+	  children_of_pid = atoi(current+14);
+	  show_all = 1;
+	  break;
 	} else if (!strncmp(current, "name=", 5)) {
 	  only_name = current+5;
 	  show_all = 1;
@@ -418,6 +475,13 @@ int main(int argc, char *argv[])
       }
       only_pid = strtol(argv[1], NULL, 10);
       opt = 1;
+    } else if (!strcmp(argv[0], "--children-of-pid")) {
+      if (argc < 2) {
+	usage(callname, stderr);
+	exit(EXIT_FAILURE);
+      }
+      children_of_pid = strtol(argv[1], NULL, 10);
+      opt = 1;
     } else if (!strcmp(argv[0], "--name")) {
       if (argc < 2) {
 	usage(callname, stderr);
@@ -449,6 +513,21 @@ int main(int argc, char *argv[])
       pidcmd = argv[1];
       opt = 1;
 
+    } else if (!strcmp (argv[0], "--lstopo-misc")) {
+      if (argc < 2) {
+	usage(callname, stderr);
+	exit(EXIT_FAILURE);
+      }
+      if (!strcmp(argv[1], "-"))
+        lstopo_misc_output = stdout;
+      else
+        lstopo_misc_output = fopen(argv[1], "w");
+      if (!lstopo_misc_output) {
+        fprintf(stderr, "Failed to open --lstopo-misc output `%s' for writing (%s)\n", argv[1], strerror(errno));
+        exit(EXIT_FAILURE);
+      }
+      opt = 1;
+
     } else if (!strcmp (argv[0], "--json-server")) {
       json_server = 1;
     } else if (!strcmp (argv[0], "--json-port")) {
@@ -522,5 +601,7 @@ int main(int argc, char *argv[])
  out_with_topology:
   hwloc_topology_destroy(topology);
  out:
+  if (lstopo_misc_output && lstopo_misc_output != stdout)
+    fclose(lstopo_misc_output);
   return err;
 }
