@@ -1,6 +1,7 @@
 /*
+ * SPDX-License-Identifier: BSD-3-Clause
  * Copyright © 2009, 2024 CNRS
- * Copyright © 2009-2024 Inria.  All rights reserved.
+ * Copyright © 2009-2025 Inria.  All rights reserved.
  * Copyright © 2009-2012 Université Bordeaux
  * Copyright © 2009-2011 Cisco Systems, Inc.  All rights reserved.
  * Copyright © 2023 Université de Reims Champagne-Ardenne.  All rights reserved.
@@ -240,7 +241,8 @@ hwloc_utils_autodetect_input_format(const char *input, int verbose)
 }
 
 static __hwloc_inline int
-hwloc_utils_enable_input_format(struct hwloc_topology *topology, unsigned long flags,
+hwloc_utils_enable_input_format(struct hwloc_topology *topology,
+				unsigned long flags __hwloc_attribute_unused,
 				const char *input,
 				struct hwloc_utils_input_format_s *input_formatp,
 				int verbose, const char *callname)
@@ -871,9 +873,10 @@ hwloc_utils_get_best_node_in_array_by_memattr(hwloc_topology_t topology, hwloc_m
                                               unsigned nbnodes, hwloc_obj_t *nodes,
                                               struct hwloc_location *initiator,
                                               unsigned long flags,
-                                              hwloc_nodeset_t best_nodeset)
+                                              hwloc_nodeset_t best_nodeset,
+                                              int verbose)
 {
-  unsigned i, j;
+  unsigned i, j, nb = 0;
   hwloc_uint64_t *values, bestvalue = 0;
   unsigned long mflags;
   int err;
@@ -886,28 +889,32 @@ hwloc_utils_get_best_node_in_array_by_memattr(hwloc_topology_t topology, hwloc_m
 
   if (mflags & HWLOC_MEMATTR_FLAG_NEED_INITIATOR) {
     /* iterate over targets, and then on their initiators */
-    for(i=0; i<nbnodes; i++) {
+    for(i=0, nb = 0; i<nbnodes; i++) {
       unsigned nbi;
       struct hwloc_location *initiators;
 
       nbi = 0;
       err = hwloc_memattr_get_initiators(topology, id, nodes[i], 0, &nbi, NULL, NULL);
-      if (err < 0)
-        goto out;
+      if (err < 0) {
+        hwloc_bitmap_zero(best_nodeset);
+        goto none;
+      }
 
       initiators = malloc(nbi * sizeof(*initiators));
       values = malloc(nbi * sizeof(*values));
       if (!initiators || !values) {
+        hwloc_bitmap_zero(best_nodeset);
         free(initiators);
         free(values);
-        goto out;
+        goto none;
       }
 
       err = hwloc_memattr_get_initiators(topology, id, nodes[i], 0, &nbi, initiators, values);
       if (err < 0) {
+        hwloc_bitmap_zero(best_nodeset);
         free(initiators);
         free(values);
-        goto out;
+        goto none;
       }
 
       for(j=0; j<nbi; j++) {
@@ -936,6 +943,8 @@ hwloc_utils_get_best_node_in_array_by_memattr(hwloc_topology_t topology, hwloc_m
         hwloc_utils__update_best_node(nodes[i], values[j],
                                       &bestvalue, best_nodeset,
                                       mflags);
+        nb++;
+        break; /* there shouldn't be multiple initiators */
       }
 
       free(initiators);
@@ -946,22 +955,54 @@ hwloc_utils_get_best_node_in_array_by_memattr(hwloc_topology_t topology, hwloc_m
     /* no initiator, just iterate over targets */
     for(i=0; i<nbnodes; i++) {
       uint64_t value;
-      if (!hwloc_memattr_get_value(topology, id, nodes[i], NULL, 0, &value))
+      if (!hwloc_memattr_get_value(topology, id, nodes[i], NULL, 0, &value)) {
         hwloc_utils__update_best_node(nodes[i], value,
                                       &bestvalue, best_nodeset,
                                       mflags);
+        nb++;
+      }
     }
   }
 
-  if ((flags & HWLOC_UTILS_BEST_NODE_FLAG_DEFAULT)
-      && hwloc_bitmap_iszero(best_nodeset)) {
-    for(i=0; i<nbnodes; i++)
-      hwloc_bitmap_set(best_nodeset, nodes[i]->os_index);
+  if (hwloc_bitmap_iszero(best_nodeset)) {
+  none:
+    if (flags & HWLOC_UTILS_BEST_NODE_FLAG_DEFAULT) {
+      if (!nb) {
+        if (verbose > 0)
+          printf("found no node with attribute values for this initiator, falling back to default\n");
+      } else {
+        fprintf(stderr, "couldn't find attribute values for all nodes, falling back to default\n");
+      }
+
+      /* try to get default nodes only */
+      hwloc_bitmap_t default_nodeset = hwloc_bitmap_alloc();
+      if (default_nodeset) {
+        err = hwloc_topology_get_default_nodeset(topology, default_nodeset, 0);
+        if (!err) {
+          hwloc_bitmap_zero(best_nodeset);
+          for(i=0; i<nbnodes; i++)
+            if (hwloc_bitmap_isset(default_nodeset, nodes[i]->os_index))
+              hwloc_bitmap_set(best_nodeset, nodes[i]->os_index);
+        }
+      }
+      free(default_nodeset);
+      /* if still empty, use all local nodes */
+      if (hwloc_bitmap_iszero(best_nodeset))
+        for(i=0; i<nbnodes; i++)
+          hwloc_bitmap_set(best_nodeset, nodes[i]->os_index);
+
+    } else {
+      if (!nb) {
+        if (verbose > 0)
+          printf("found no node with attribute values for this initiator\n");
+      } else {
+        fprintf(stderr, "couldn't find attribute values for all nodes, returning none\n");
+      }
+    }
   }
   return 0;
 
  out:
-  hwloc_bitmap_zero(best_nodeset);
   return -1;
 }
 
@@ -1163,6 +1204,7 @@ hwloc_utils_parse_local_numanode_flags(char *str) {
   struct hwloc_utils_parsing_flag possible_flags[] = {
     HWLOC_UTILS_PARSING_FLAG(HWLOC_LOCAL_NUMANODE_FLAG_LARGER_LOCALITY),
     HWLOC_UTILS_PARSING_FLAG(HWLOC_LOCAL_NUMANODE_FLAG_SMALLER_LOCALITY),
+    HWLOC_UTILS_PARSING_FLAG(HWLOC_LOCAL_NUMANODE_FLAG_INTERSECT_LOCALITY),
     HWLOC_UTILS_PARSING_FLAG(HWLOC_LOCAL_NUMANODE_FLAG_ALL)
   };
 
